package com.mom.jcrperf.jackrabbit;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import javax.jcr.Session;
import javax.jcr.SimpleCredentials;

import org.apache.jackrabbit.core.RepositoryImpl;
import org.apache.jackrabbit.core.SessionImpl;
import org.apache.jackrabbit.core.config.RepositoryConfig;
import org.apache.jackrabbit.core.fs.db.DbFileSystem;

import com.mom.jcrperf.AbstractTestableRepository;

/**
 * An abstract base class for Jackrabbit.
 * Subclasses need to be created for specific versions (1.6, 2.x etc)
 * @author duncan
 *
 */
public abstract class TestableJackrabbitRepository extends AbstractTestableRepository {
	
	protected RepositoryConfig config;
	
	protected String repositoryDir;
	
	public void setRepositoryDir(String dir) {
		repositoryDir = dir;
	}


	public void initialise() throws Exception {
		File f = new File(repositoryDir);

		if (!f.exists()) {
			throw new Exception("Repository directory " + repositoryDir + " does not exist");
		}

		f = new File(repositoryDir + "/repository.xml");

		if (!f.exists()) {
			throw new Exception("Repository directory " + repositoryDir + " does not contain a repository.xml file");
		}

		// Delete existing workspaces
		File w = new File(repositoryDir+"/workspaces");
		deleteDir(w);
		
		config = RepositoryConfig.create(repositoryDir + "/repository.xml", repositoryDir);
		DbFileSystem fileSystem = (DbFileSystem) config.getFileSystem();
		
		Connection connection = getConnection(fileSystem.getDriver(), fileSystem.getUrl(),
				fileSystem.getUser(), fileSystem.getPassword());

		cleanDatabase(fileSystem, connection);

		// Now recreate the repository
		repository = RepositoryImpl.create(config);
		credentials = new SimpleCredentials("admin", "admin".toCharArray());

		// Register our custom nodetypes
		Session session = repository.login(credentials);
		InputStream customNodetypes = new FileInputStream(repositoryDir + "/custom_nodetypes.xml");
        		 ((SessionImpl)session).getNodeTypeManager().registerNodeTypes(customNodetypes, "text/xml");


	}

	public void shutdown() {
		// TODO Auto-generated method stub
		
	}

    public static void cleanDatabase(DbFileSystem fileSystem, Connection connection) throws SQLException {
		// Delete all existing tables
		Statement s = connection.createStatement();
		boolean mysql = fileSystem.getDriver().contains("mysql");
		dropTable(s, mysql, "DATASTORE");
		dropTable(s, mysql, "REP_FSENTRY");

		dropTable(s, mysql, "SECURITY_BINVAL");
		dropTable(s, mysql, "SECURITY_BUNDLE");
		dropTable(s, mysql, "SECURITY_FSENTRY");
		dropTable(s, mysql, "SECURITY_NAMES");
		dropTable(s, mysql, "SECURITY_REFS");

		dropTable(s, mysql, "SYSTEM_BINVAL");
		dropTable(s, mysql, "SYSTEM_BUNDLE");
		dropTable(s, mysql, "SYSTEM_FSENTRY");
		dropTable(s, mysql, "SYSTEM_NAMES");
		dropTable(s, mysql, "SYSTEM_REFS");

		dropTable(s, mysql, "VERSION_BINVAL");
		dropTable(s, mysql, "VERSION_BUNDLE");
		dropTable(s, mysql, "VERSION_FSENTRY");
		dropTable(s, mysql, "VERSION_NAMES");
		dropTable(s, mysql, "VERSION_REFS");
    }
    
    public static void dropTable(Statement s, boolean mysql, String table) throws SQLException {
    	String query;
    	
    	if(mysql) {
    		 query = "DROP TABLE IF EXISTS "+table;
		} else {
			 query = "IF EXISTS(SELECT TABLE_NAME FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME = '"+table+"') DROP TABLE "+table;
		}
    	s.execute(query);
    }
    
    public Map<String, Map<String, String>> getConfiguration() {
		
		Properties properties = config.getVersioningConfig().getPersistenceManagerConfig().getParameters();
		
		Map<String, Map<String, String>> configs = super.getConfiguration();

		Map<String, String> pm = new HashMap<String, String>();
		for(String p : properties.stringPropertyNames()) {
			String v = properties.getProperty(p);
			pm.put(p, v);
		}
		configs.put("persistenceManager", pm);
		
		return configs;
    }

}
