package com.mom.jcrperf;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Map;

import javax.jcr.Session;
import javax.jcr.SimpleCredentials;

import org.modeshape.common.collection.Problem;
import org.modeshape.jcr.JcrConfiguration;
import org.modeshape.jcr.JcrEngine;
import org.modeshape.jcr.api.AnonymousCredentials;
import org.modeshape.jcr.api.nodetype.NodeTypeManager;

/**
 * Custom TestableRepository for connecting to Modeshape
 * @author duncan
 *
 */

public class TestableRepository extends AbstractTestableRepository  {
	
	/**
	 * Location of the configuration directory containing the files
	 * repository.xml
	 * custom_nodetypes.xml
	 */
	private String configurationDir;
	/**
	 * Name of the repository to use for this test
	 */
	public static final String repositoryName = "modeshape_test_repo";
	
	public String getRepositoryName() {
		return repositoryName;
	}

	private JcrEngine engine;
	
	
	public String getConfigurationDir() {
		return configurationDir;
	}

	public void setConfigurationDir(String configurationDir) {
		this.configurationDir = configurationDir;
	}

	/**
	 * 
	 */
    
    public Map<String, Map<String, String>> getConfiguration() {
		
		Map<String, Map<String, String>> configs = super.getConfiguration();
		
		return configs;
    }

	@Override
	public void initialise() throws Exception {
        // Load the configuration from the repository.xml file
		File file = new File(configurationDir+"/repository.xml"); 
        JcrConfiguration configuration = new JcrConfiguration();
        configuration.loadFrom(file);

        // Now create the JCR engine ...
        engine = configuration.build();
        engine.start();

        if (engine.getProblems().hasProblems()) {
            for (Problem problem : engine.getProblems()) {
                System.err.println(problem.getMessageString());
            }
            throw new RuntimeException("Could not start due to problems");
        }
        
        repository = engine.getRepository(repositoryName);
		credentials = new AnonymousCredentials();

		// Register our custom nodetypes
		Session session = repository.login(credentials);
		File customNodes = new File(configurationDir + "/custom_nodetypes.xml");
		if(customNodes.canRead()) {
			InputStream customNodetypes = new FileInputStream(customNodes);
			NodeTypeManager manager = (NodeTypeManager)session.getWorkspace().getNodeTypeManager();
	        manager.registerNodeTypes(customNodetypes, true);
		} else {
			System.err.println("WARNING: Custom node types not found at "+customNodes.getPath());
		}
    }

	@Override
	public void shutdown() {
		engine.shutdown();
	}

}
