/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.mom.jcrperf.tests;

import java.util.Calendar;

import javax.jcr.ItemExistsException;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.PathNotFoundException;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.lock.LockException;
import javax.jcr.nodetype.ConstraintViolationException;
import javax.jcr.nodetype.NoSuchNodeTypeException;
import javax.jcr.version.VersionException;

/**
 * Builds a small tree of objects with a large number of properties in order to test the bundle cache
 * 
 */
public class VersionedBundleTest extends PerformanceTest {

    private Session session;

    private Node node;

    public void beforeSuite() throws RepositoryException {
        session = getRepository().login(getCredentials());
        node = session.getRootNode().addNode("testnode", "mtm:pathway");
		
        node.setProperty("mtm:externalId", 1);
        node.setProperty("mtm:name", "testnode (pathway)");
        node.setProperty("mtm:createdBy", "duncan");
        node.setProperty("mtm:creationTimestamp", Calendar.getInstance());
        node.setProperty("mtm:modifiedBy", "duncan");
        node.setProperty("mtm:modificationTimestamp", Calendar.getInstance());
        node.setProperty("mtm:description", "a test node");
        session.save();
    }

    public void beforeTest() throws RepositoryException {
    }

    public void runTest() throws Exception {
    	buildTree(node, 1);
    	touchTree(node);
    }
    
    public void buildTree(Node parent, int depth) throws ItemExistsException, PathNotFoundException, NoSuchNodeTypeException, LockException, VersionException, ConstraintViolationException, RepositoryException {
    	if(depth < 1) {
    		return;
    	}
    	
    	for(int i=0; i<60; i++) {
    		String name = "L"+depth+"_C"+i;
    		
    		Node child = parent.addNode(name, "mtm:page");
    		child.addMixin("mix:versionable");
    		
    		child.setProperty("mtm:deleted", false);
    		//child.setProperty("mtm:lastClinicalReviewTimestamp", new Date().toString());
    		child.setProperty("mtm:externalId", i);
    		child.setProperty("mtm:previousId", name);
    		child.setProperty("mtm:name", name);
    		child.setProperty("mtm:createdBy", "duncan");
    		child.setProperty("mtm:creationTimestamp", Calendar.getInstance());
    		child.setProperty("mtm:modifiedBy", "duncan");
    		child.setProperty("mtm:modificationTimestamp", Calendar.getInstance());
    		child.setProperty("mtm:description", "a test node");
    		child.setProperty("mtm:shortName", "a test node");
    		// Add a load of properties
    		//for(int j=0; j<1024; j++) {
    		
    			//child.setProperty("prop_"+j, "This is the value of property number "+j);
    		//}
    		//child.setProperty("touched", "no");
    		
    		session.save();
    		buildTree(child, depth-1);
      	}
    }
    
    public void touchTree(Node parentNode) throws RepositoryException {
    	for(NodeIterator it = parentNode.getNodes(); it.hasNext();) {
    		Node child = it.nextNode();
    		child.checkout();
    		child.setProperty("mtm:name", "touched");
//    		for(int j=0; j<1024; j++) {
  //  			child.setProperty("prop_"+j, "This is the changed value of property number "+j);
    //		}
    	//	child.setProperty("touched", "yes");
    		child.save();
    		child.checkin();
    		touchTree(child);
    	}
    }

    public void afterTest() throws RepositoryException {
    }

    public void afterSuite() throws RepositoryException {
        session.getRootNode().getNode("testnode").remove();
        session.save();
        session.logout();
    }

}

