/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.mom.jcrperf.tests;

import javax.jcr.ItemExistsException;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.PathNotFoundException;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.lock.LockException;
import javax.jcr.nodetype.ConstraintViolationException;
import javax.jcr.nodetype.NoSuchNodeTypeException;
import javax.jcr.version.VersionException;

public class VersionedTreeTest extends PerformanceTest {

    private Session session;

    private Node node;

    public void beforeSuite() throws RepositoryException {
        session = getRepository().login(getCredentials());
        node = session.getRootNode().addNode("testnode", "nt:unstructured");
		session.save();
    }

    public void beforeTest() throws RepositoryException {
    }

    public void runTest() throws Exception {
    	buildTree(node, 3);
    	touchTree(node);
    }
    
    public void buildTree(Node parent, int depth) throws ItemExistsException, PathNotFoundException, NoSuchNodeTypeException, LockException, VersionException, ConstraintViolationException, RepositoryException {
    	if(depth < 1) {
    		return;
    	}
    	
    	for(int i=0; i<3; i++) {
    		String name = "L"+depth+"_C"+i;
    		
    		Node child = parent.addNode(name, "nt:unstructured");
    		child.addMixin("mix:versionable");
    		child.setProperty("touched", "no");
    		
    		session.save();
    		buildTree(child, depth-1);
      	}
    }
    
    public void touchTree(Node parentNode) throws RepositoryException {
    	for(NodeIterator it = parentNode.getNodes(); it.hasNext();) {
    		Node child = it.nextNode();
    		child.checkout();
    		child.setProperty("touched", "yes");
    		child.save();
    		child.checkin();
    		touchTree(child);
    	}
    }

    public void afterTest() throws RepositoryException {
    }

    public void afterSuite() throws RepositoryException {
        session.getRootNode().getNode("testnode").remove();
        session.save();
        session.logout();
    }

}

